//
// @author vay
// @email hxhvay@126.com
// @date 2020-06-04
//

import 'package:vaycore/http.dart';

import 'package:vaycore/utils.dart';

/// 分页请求基类（简化分页流程处理）
abstract class BasePaging<T> {
  final String apiUrl;
  final int count;
  final int _initStart;

  int start;
  bool _locker = false;
  bool _moreData = true;
  List<T> _accumulator = [];

  List<T> get accumulator => _accumulator;

  BasePaging(
    this.apiUrl, {
    this.count = 10,
    this.start = 0,
  })  : assert(!StringUtils.isEmpty(apiUrl)),
        this._initStart = start;

  void _prepareRefresh() {
    this.start = this._initStart;
    this._moreData = true;
  }

  /// 刷新数据
  Future<PageResult<T>> refreshData() async {
    if (!_getLocker()) {
      return null;
    }
    _prepareRefresh();
    PageResult data = await this._actualGetData();
    _releaseLocker();
    return data;
  }

  /// 获取更多数据
  Future<PageResult<T>> getMoreData() async {
    if (!this._moreData) {
      return null;
    }
    if (!_getLocker()) {
      return null;
    }
    PageResult data = await this._actualGetData();
    _releaseLocker();
    return data;
  }

  /// 构建请求方式
  HttpMethod buildMethod() => HttpMethod.GET;

  /// 构建请求头
  Map<String, Object> buildHeader() => {};

  /// 构建请求数据
  Map<String, Object> buildData(int start, int count) {
    return {"start": start, "count": count};
  }

  /// 构建下一页的start（请求返回结果后回调）
  int buildNextStart(int start) {
    return start += this.count;
  }

  /// 数据请求
  Future<PageResponse> internalRequest(
    String url,
    HttpMethod method,
    Map<String, Object> data,
    Map<String, Object> header,
  );

  Future<PageResult<T>> _actualGetData() async {
    PageResponse response = await _doRequest();
    if (response == null || response.total == 0) {
      // 请求失败或无数据，都把之前请求到的列表数据返回
      return PageResult<T>(
        accumulator: this._accumulator,
      );
    }

    // 如果是刷新，先把列表数据清空
    if (this.start == this._initStart) {
      _clearAccumulate();
    }

    this._moreData = BasePaging._hasMoreData(response.totalPage, response.page);
    if (this._moreData) {
      this.start = buildNextStart(this.start);
    }
    _accumulate(response.items);
    return PageResult(
      empty: false,
      items: response.items,
      moreData: this._moreData,
      accumulator: this._accumulator,
    );
  }

  /// 执行网络请求
  Future<PageResponse<T>> _doRequest() {
    return internalRequest(
      this.apiUrl,
      buildMethod(),
      buildData(this.start, this.count),
      buildHeader(),
    );
  }

  _accumulate(List<T> items) {
    this._accumulator.addAll(items);
  }

  _clearAccumulate() {
    this._accumulator.clear();
  }

  static _hasMoreData(totalPage, pageNum) {
    return pageNum < totalPage - 1;
  }

  _getLocker() {
    if (this._locker) {
      return false;
    }
    this._locker = true;
    return true;
  }

  _releaseLocker() {
    this._locker = false;
  }
}

/// 分页请求响应
class PageResponse<T> {
  /// 总数量
  final int total;

  /// 每页数量
  final int count;

  /// 当前页
  final int page;

  /// 总页数
  final int totalPage;

  /// 列表数据
  final List<T> items;

  const PageResponse({
    this.total,
    this.count,
    this.page,
    this.totalPage,
    this.items,
  });
}

/// 分页返回数据结果
class PageResult<T> {
  final bool empty;
  final List<T> items;
  final bool moreData;
  final List<T> accumulator;

  const PageResult({
    this.empty = true,
    this.items = const [],
    this.moreData = false,
    this.accumulator = const [],
  });

  int get itemCount => items?.length ?? 0;

  /// accumulator列表的数据数量
  int get totalCount => accumulator?.length ?? 0;
}
