//
// @author vay
// @email hxhvay@126.com
// @date 2020-07-06
//

import 'package:vaycore/core.dart';
import 'package:vaycore/dartin.dart';
import 'package:vaycore/http.dart';
import 'package:vaycore/modules.dart';
import 'package:vaycore/mvvm.dart';

/// VayCore引擎接口
abstract class IVCEngine extends VCComponent {
  /// 构造方法
  IVCEngine() {
    _loadComponent();
  }

  /// 加载组件
  void _loadComponent() {
    List<VCComponentBuilder> singleComponents = [
      ..._defaultSingleComponents(),
      ...registerSingleComponents(),
    ];
    List<VCComponentBuilder> factoryComponents = [
      ..._defaultFactoryComponents(),
      ...registerFactoryComponents(),
    ];
    // 开始注入模块
    startDartIn([
      _doRegisterSingleComponents(singleComponents),
      _doRegisterFactoryComponents(factoryComponents),
    ]);
  }

  /// 默认注册的单例组件
  List<VCComponentBuilder> _defaultSingleComponents() {
    return [
      newComponentBuilder((params) => VCTheme()),
      newComponentBuilder((params) => VCLocale()),
    ];
  }

  /// 默认注册的工厂组件
  List<VCComponentBuilder> _defaultFactoryComponents() {
    return [
      newComponentBuilder((params) => VoidProvider()),
      newComponentBuilder((params) => VCThemeHolder()),
      newComponentBuilder((params) => VCLocaleHolder()),
    ];
  }

  /// 创建组件构建器
  VCComponentBuilder newComponentBuilder<T extends VCComponent>(
      VCComponentBuilderCallback<T> callback) {
    return VCComponentBuilder<T>(callback);
  }

  @override
  onInit() {}

  /// 注册单例组件
  Module _doRegisterSingleComponents(List<VCComponentBuilder> components) {
    List<DartIn> list = List.generate(components.length, (index) {
      VCComponentBuilder builder = components[index];
      return builder._buildDartIn(singleton: true);
    });
    return Module(list);
  }

  /// 注册工厂组件
  Module _doRegisterFactoryComponents(List<VCComponentBuilder> components) {
    List<DartIn> list = List.generate(components.length, (index) {
      return components[index]._buildDartIn();
    });
    return Module(list);
  }

  /// 网络请求日志配置
  HttpLogOptions buildHttpLogOptions() {
    return HttpLogOptions();
  }

  /// 屏幕适配对应的屏幕宽度（UI设计图的宽度）
  double get adapterWidth => 375;

  /// 主题业务处理
  IVCThemeHolder buildThemeHolder();

  /// 国际化业务处理
  IVCLocaleHolder buildLocaleHolder();

  /// 注册单例组件
  List<VCComponentBuilder> registerSingleComponents();

  /// 注册工厂组件
  List<VCComponentBuilder> registerFactoryComponents();

  /// 构建根组件
  dynamic buildRootWidget();
}

/// 组件构建方法
typedef VCComponentBuilderCallback<T> = T Function(dynamic params);

/// 组件构建器
class VCComponentBuilder<T extends VCComponent> {
  /// 构建器回调
  final VCComponentBuilderCallback<T> _builderCallback;

  VCComponentBuilder(this._builderCallback);

  /// 构建组件
  DartIn<T> _buildDartIn({bool singleton = false}) {
    return singleton
        ? single<T>(({params}) => _buildComponent(params))
        : factory<T>(({params}) => _buildComponent(params));
  }

  T _buildComponent(dynamic params) {
    VCComponent component = _builderCallback(params);
    component.onInit();
    return component;
  }
}
