//
// @author vay
// @email hxhvay@126.com
// @date 2020-07-07
//

import 'package:flutter/material.dart';
import 'package:vaycore/dartin.dart';
import 'package:vaycore/mvvm.dart';
import 'package:vaycore/src/modules/locale/i_vc_locale_holder.dart';
import 'package:vaycore/vaycore.dart';

/// 国际化状态
class VCLocale extends BaseProvider {
  /// 默认支持的语言列表
  static const DEFAULT_SUPPORT_LOCALES = [
    const Locale('zh', 'CN'),
    const Locale('en', 'US'),
  ];

  static VCLocale get _instance => inject();

  /// 国际化业务处理
  IVCLocaleHolder _localeHolder;

  /// 支持的语言列表
  static List<Locale> get supportLocales {
    if (_instance._localeHolder == null) {
      return DEFAULT_SUPPORT_LOCALES;
    }
    return _instance._localeHolder.supportLocales;
  }

  /// 获取当前用户的APP语言配置Locale类，如果为null，则语言跟随系统语言
  static Locale getLocale() {
    if (_instance._localeHolder == null) {
      return supportLocales.first;
    }
    return _instance._localeHolder.readLocale();
  }

  /// 设置当前用户的APP语言
  static setLocale(Locale locale) {
    if (_instance._localeHolder == null) {
      return;
    }
    _instance._localeHolder.saveLocale(locale);
    _instance.notifyListeners();
  }

  @override
  onInit() {
    this._localeHolder = core.engine.buildLocaleHolder();
  }

  /// 简化国际化回调处理
  ///
  /// ```
  /// MaterialApp(
  ///   ...
  ///   localeResolutionCallback:
  ///               (Locale locale, Iterable<Locale> supportedLocales) =>
  ///                   core.options.locale.buildLocaleResolutionCallback(
  ///                       localeModel.locale, locale, supportedLocales),
  /// )
  /// ```
  static Locale localeResolutionCallback(
    Locale locale,
    Iterable<Locale> supportedLocales,
  ) {
    Locale storageLocale = _instance._localeHolder.readLocale();
    if (storageLocale != null) {
      //如果已经选定语言，则不跟随系统
      return storageLocale;
    } else {
      Locale resultLocale;
      // APP语言跟随系统语言，如果系统语言不是中文简体或美国英语，
      // 则默认使用支持列表的第一个语言
      if (supportedLocales.contains(locale)) {
        resultLocale = locale;
      } else {
        resultLocale = supportLocales.first;
      }
      setLocale(resultLocale);
      return resultLocale;
    }
  }

  /// 获取国际化字符串
  static String getString<T>(T ids, [List<dynamic> params]) {
    return _instance._localeHolder.getString(ids, params);
  }
}
