//
// @author vay
// @email hxhvay@126.com
// @date 2020-05-16
//

import 'package:vaycore/mvvm.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';
import 'package:vaycore/dartin.dart';

import 'package:vaycore/vaycore.dart';

/// 无状态页面基类 [BaseStatelessPage]
abstract class BaseStatelessPage<T extends BaseProvider> extends StatelessWidget
    with ConsumerHelper<T> {
  final T mProvider;

  /// 构造函数
  ///
  /// [params] 代表注入ViewModel[mProvider]时所需的参数，需按照[mProvider]的构造方法顺序赋值
  BaseStatelessPage({Key key, List<dynamic> params})
      : mProvider = inject<T>(params: params),
        super(key: key);

  Widget buildContent(BuildContext context);

  @override
  Widget build(BuildContext context) {
    if (core.context == null) {
      core.context = context;
    }
    return changeNotifier<T>(
      context,
      provider: mProvider,
      builder: (ctx) => buildContent(context),
    );
  }
}

/// 有状态的页面
///
/// 可用此模板添加到Android Studio的template配置，方法增加页面
/// ```
///  class $NAME$ extends BaseStatefulPage<$PROVIDER$> {
///    @override
///    _$NAME$State buildState() => _$NAME$State();
///  }
///
///  class _$NAME$State extends BaseState<$NAME$, $PROVIDER$> {
///    @override
///    Widget buildContent(BuildContext context) {
///      return Container($END$);
///    }
///  }
/// ```
///
abstract class BaseStatefulPage<T extends BaseProvider> extends StatefulWidget {
  final T provider;

  /// 构造函数
  ///
  /// [params] 代表注入ViewModel[provider]时所需的参数，需按照[provider]的构造方法顺序赋值
  BaseStatefulPage({Key key, List<dynamic> params})
      : provider = inject<T>(params: params),
        super(key: key);

  BaseState<BaseStatefulPage<T>, T> createState() {
    BaseState state = buildState();
    state.provider = this.provider;
    this.provider.state = state;
    return state;
  }

  /// 构建[BaseState]对象
  BaseState buildState();
}

/// 有状态页面的状态类
abstract class BaseState<T extends BaseStatefulPage, P extends BaseProvider>
    extends State<T> with AutomaticKeepAliveClientMixin, ConsumerHelper<P> {
  P provider;

  @override
  bool get wantKeepAlive => false;

  Widget buildContent(BuildContext context);

  @override
  @mustCallSuper
  void initState() {
    // 用于初始化一些变量
    provider.onStart();
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    if (core.context == null) {
      core.context = context;
    }
    return changeNotifier<P>(
      context,
      provider: provider,
      builder: buildContent,
    );
  }
}

mixin ConsumerHelper<P extends BaseProvider> {
  /// 注册需要通知刷新的组件
  Widget consumer(WidgetBuilder builder) {
    return Consumer<P>(
      builder: (BuildContext context, P provide, Widget widget) {
        return builder(context);
      },
    );
  }

  /// 注册需要通知刷新的组件
  Widget consumerType<T>(WidgetBuilder builder) {
    return Consumer<T>(
      builder: (BuildContext context, T provide, Widget widget) {
        return builder(context);
      },
    );
  }

  /// 注册需要通知刷新的组件
  Widget consumerType2<A, B>(WidgetBuilder builder) {
    return Consumer2<A, B>(
      builder: (
        BuildContext context,
        A provideA,
        B providerB,
        Widget widget,
      ) {
        return builder(context);
      },
    );
  }

  /// 注册provider节点
  Widget changeNotifier<N extends BaseProvider>(
    BuildContext context, {
    @required N provider,
    @required WidgetBuilder builder,
  }) {
    return ChangeNotifierProvider<N>.value(
      value: provider,
      child: builder(context),
    );
  }

  /// 注册provider节点并注册通知刷新
  Widget changeNotifierWithConsumer<N extends BaseProvider>({
    @required N provider,
    @required WidgetBuilder builder,
  }) {
    return ChangeNotifierProvider<N>.value(
      value: provider,
      child: consumerType<N>((context) => builder(context)),
    );
  }
}
